@@ -1772,13 +1772,12 @@ gdl_dock_item_set_tablabel (GdlDockItem *item,
                                                   NULL, item);
             g_object_set (item->_priv->tab_label, "item", NULL, NULL);
         }
-        gtk_widget_unref (item->_priv->tab_label);
+        g_object_unref (item->_priv->tab_label);
         item->_priv->tab_label = NULL;
     }
     
     if (tablabel) {
-        gtk_widget_ref (tablabel);
-        gtk_object_sink (GTK_OBJECT (tablabel));
+        g_object_ref_sink (G_OBJECT (tablabel));
         item->_priv->tab_label = tablabel;
         if (GDL_IS_DOCK_TABLABEL (tablabel)) {
             g_object_set (tablabel, "item", item, NULL);
@@ -1789,6 +1788,23 @@ gdl_dock_item_set_tablabel (GdlDockItem *item,
     }
 }
 
+/**
+ * gdl_dock_item_get_grip:
+ * @item: The dock item from which to to get the grip of.
+ * 
+ * This function returns the dock item's grip label widget.
+ * 
+ * Returns: Returns the current label widget.
+ **/
+GtkWidget *
+gdl_dock_item_get_grip(GdlDockItem *item)
+{
+    g_return_if_fail (item != NULL);
+    g_return_val_if_fail (GDL_IS_DOCK_ITEM (item), NULL);
+
+    return item->_priv->grip;
+}
+
 /**
  * gdl_dock_item_hide_grip:
  * @item: The dock item to hide the grip of.
@@ -1908,8 +1924,7 @@ gdl_dock_item_hide_item (GdlDockItem *item)
                           "floatx", x,
                           "floaty", y,
                           NULL));
-        g_object_ref (item->_priv->ph);
-        gtk_object_sink (GTK_OBJECT (item->_priv->ph));
+        g_object_ref_sink (item->_priv->ph);
     }
     
     gdl_dock_object_freeze (GDL_DOCK_OBJECT (item));
@@ -2052,8 +2067,7 @@ gdl_dock_item_set_default_position (GdlDockItem   *item,
 
     if (reference && GDL_DOCK_OBJECT_ATTACHED (reference)) {
         if (GDL_IS_DOCK_PLACEHOLDER (reference)) {
-            g_object_ref (reference);
-            gtk_object_sink (GTK_OBJECT (reference));
+            g_object_ref_sink (reference);
             item->_priv->ph = GDL_DOCK_PLACEHOLDER (reference);
         } else {
             item->_priv->ph = GDL_DOCK_PLACEHOLDER (
@@ -2061,8 +2075,7 @@ gdl_dock_item_set_default_position (GdlDockItem   *item,
                               "sticky", TRUE,
                               "host", reference,
                               NULL));
-            g_object_ref (item->_priv->ph);
-            gtk_object_sink (GTK_OBJECT (item->_priv->ph));
+            g_object_ref_sink (item->_priv->ph);
         }
     }
 }
