@@ -36,11 +36,6 @@
 
 #include <gtk/gtk.h>
 
-#if HAVE_GNOME
-#include <gconf/gconf-client.h>
-#include <libgnome/gnome-gconf.h>
-#endif
-
 static void gdl_switcher_set_property  (GObject            *object,
                                         guint               prop_id,
                                         const GValue       *value,
